/*
 * Peony-Qt's Library
 *
 * Copyright (C) 2020, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Yue Lan <lanyue@kylinos.cn>
 *
 */

#ifndef TOOLBARACTIONPLUGINIFACE_H
#define TOOLBARACTIONPLUGINIFACE_H

#include "plugin-iface.h"
#include <QAction>

#define ToolBarActionPluginIface_iid "org.ukui.peony-qt.plugin-iface.ToolBarActionPluginInterface"

namespace Peony {

class ToolBarExtensionActionIface : public QAction
{
    Q_OBJECT

    ~ToolBarExtensionActionIface() {}

public Q_SLOTS:
    virtual void excuteAction(const QString &directoryUri,
                              const QStringList &selectedUris) = 0;
};

class ToolBarActionPluginIface : public PluginInterface
{

public:
    ~ToolBarActionPluginIface() {}

    virtual ToolBarExtensionActionIface *create() = 0;
};

}

Q_DECLARE_INTERFACE(Peony::ToolBarActionPluginIface, ToolBarActionPluginIface_iid)

#endif // TOOLBARACTIONPLUGINIFACE_H
